/** @file syn_supnorm.h Supremal normal sublanguage */

/* FAU Discrete Event Systems Library (libfaudes)

   Copyright (C) 2009  Sebastian Perk, Thomas Moor

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

 
#ifndef FAUDES_SUPNORM_H
#define FAUDES_SUPNORM_H

#include "corefaudes.h"
#include <stack>

namespace faudes {

/**
 * ConcatenateFullLanguage: concatenate Sigma* to language
 * marked by rGen. Less expensive than using
 * LanguageConcatenate() to concatenate Sigma*, as no
 * additional nondeterminism is caused.
 * Used in SupNorm().
 * Method:
 * Transitions starting from marked states are erased.
 * Remaining accessible marked states are provided with
 * Sigma-selfloops.
 * Determinism:
 * Result can be nondeterministic only if parameter is
 * nondeterministic.
 *
 * @param rGen
 *   generator marking the language to be concatenated with Sigma*
 * 
 *
 */
void ConcatenateFullLanguage(vGenerator& rGen);

/**
 * NormalityConsistencyCheck: Consistency
 * check for normality input data. Used e.g. in IsNormal(),
 * IsNormalAlt() and SupNorm(). See exceptions.
 *
 * @param rL
 *   generator of language L
 * @param rOAlph
 *   observable alphabet
 * @param rK
 *   generator of language K
 *
 * @exception Exception
 *   - nondeterministic parameter(s) (id: 101)
 *   - rOAlph not subset of rL.Alphabet() (id: 100)
 *   - Alphabets of generators don't match (id: 100)
 *   - K is not subset of L (id 0)
 * 
 *
 */
void NormalityConsistencyCheck(
  const Generator& rL,
  const EventSet& rOAlph,
  const Generator& rK);

/**
 * IsNormal: checks normality of a language K generated by
 * rK wrt a language L generated by rL and the subset of
 * observable events rOAlph. This is done by checking if the
 * following equality holds:
 * pinv(p(K)) intersect L = K
 * todo(?): check for efficient algorithm replacing above
 * formula that returns false immediately after having
 * found a non-normal string -> IsNormalFast()
 * todo: implement test routines, verify correctness
 * todo: compare speed with IsNormalAlt
 *
 * @param rL
 *   generator of language L
 * @param rOAlph
 *   observable alphabet
 * @param rK
 *   generator of language K
 * 
 * @return
 *   true if K is normal w.r.t. L and OAlph
 *
 * @exception Exception
 *   - thrown by NormalityConsistencyCheck()
 * 
 * @ingroup SynthesisPlugIn
 *
 */
bool IsNormal(
  const Generator& rL,
  const EventSet& rOAlph,
  const Generator& rK);

/**
 * IsNormalAlt: Alternative implementation of IsNormal().
 * using parallel composition instead of invProject and set
 * intersection. Checks normality of a language K generated by
 * rK w.r.t. a language L generated by rL and the subset of
 * observable events rOAlph. This is done by checking if the
 * following equality holds:
 * p(K) || L = K
 * todo(?): check for efficient algorithm replacing above
 * formula that returns false immediately after having
 * found a non-normal string -> IsNormalFast()
 * todo: implement test routines, verify correctness
 * todo: compare speed with IsNormal
 * 
 * @param rL
 *   generator of language L
 * @param rOAlph
 *   observable alphabet
 * @param rK
 *   generator of language K
 * 
 * @return
 *   true if K is normal w.r.t. L and the observable
 *   alphabet
 *
 * @exception Exception
 *   - thrown by NormalityConsistencyCheck()
 * 
 *
 */
bool IsNormalAlt(
  const Generator& rL,
  const EventSet& rOAlph,
  const Generator& rK);


/** 
 * IsNormal wrapper.  
 * Wrapper for convenient access via the run-time interface.
 */
bool IsNormal(const cGenerator& rPlantGen, const vGenerator& rSupCandGen);


/**
 * SupNorm: compute supremal normal sublanguage.
 *
 * SupNorm calculates the supremal sublanguage
 * of the closed language K (prefix closure of the language
 * marked by rK) that is normal w.r.t. the closed language L
 * (prefix closure of the language marked by rL) and the set
 * of observable events.
 *
 * Method: The supremal normal sublanguage is computed
 * according to the Lin-Brandt-Formula:
 * supnorm(K)wrt(L)=K-Pinv[P(L-K)]
 *
 * SupNorm returns false on empty result.
 *
 * Parameters have to be deterministic, result is deterministic.
 *
 * Note: rL is not const, as the prefix closure is taken during
 * computation. However, its original marked language is restored
 * before return.
 * See also SupNormClosed and supnorm_tutorial.cpp
 *
 * @param rL
 *   generates the closed language L=L(rL)
 * @param rOAlph
 *   observable alphabet
 * @param rK
 *   generates the closed language K=L(rK)
 * @param rResult
 *   generates the supremal normal
 *   sublanguage, where Lm(rResult) is intersection of L(rResult) and Lm(rL)
 *   (marking consistent to rL)
 * 
 * @return
 *   true for nonempty result
 *
 * @exception Exception
 *   - Alphabets of generators don't match (id 500)
 *   - rOAlph not subset of rL.Alphabet() (id 506)
 *   - K is not subset of L. (id 0)
 *
 * @ingroup SynthesisPlugIn
 * 
 *
 */
bool SupNorm(
  Generator& rL,
  const EventSet& rOAlph,
  const Generator& rK, 
  Generator& rResult);


/**
 * SupNormClosed - compute supremal normal and closed
 * sublanguage. 
 *
 * SupNormClosed calculates the supremal sublanguage
 * of the closed language K (prefix closure of the language
 * marked by rK) that is closed  and normal w.r.t. the closed language
 * L (prefix closure of the language marked by rL) and the set
 * of observable events.
 *
 * Method: The supremal normal sublanguage is computed
 * according to the Lin-Brandt-Formula:
 * supnormclosed(K)wrt(L)=K-Pinv[P(L-K)]Sigma*
 * The difference to supnorm lies in the concatenation of Sigma*.
 * SupNormClosed returns false on empty result.
 *
 * Parameters have to be deterministic, result is deterministic.
 *
 * Note: rL is not const, as the prefix closure is taken during
 * computation. However, its original marked language is restored
 * before return.
 * See also supnorm_tutorial.cpp
 * todo: check for efficient algorithm replacing above formula
 *
 * @param rL
 *   generates the closed language L=L(rL)
 * @param rOAlph
 *   observable alphabet
 * @param rK
 *   generates the closed language K=L(rK)
 * @param rResult
 *   generates the supremal normal and closed
 *   sublanguage, where Lm(rResult) is intersection of L(rResult) and Lm(rL)
 *   (marking consistent to rL)
 * 
 * @return
 *   true for nonempty result
 *
 * @exception Exception
 *   - Alphabets of generators don't match (id 500)
 *   - rOAlph not subset of rL.Alphabet() (id 506)
 *   - K is not subset of L. (id 0)
 *
 * @ingroup SynthesisPlugIn
 * 
 *
 */
bool SupNormClosed(
  Generator& rL,
  const EventSet& rOAlph,
  const Generator& rK, 
  Generator& rResult);


/**
 * SupPrefixClosed: supremal closed sublanguage of K by cancelling
 * all tranistions leading to a non-marked state.
 * Returns false on empty result.
 * todo: implement test routines, verify correctness
 *
 * @param rK
 *   marks the (not necessarily closed) language K=Lm(rK)
 * @param rResult
 *   generates and marks the supremal closed sublanguage,
 *   where L(rResult)=Lm(rResult)
 * 
 * @return
 *   true for nonempty result
 *
 * @exception Exception
 *   - todo: check determinism of rK
 * 
 *
 */
bool SupPrefixClosed(
  const Generator& rK,
	Generator& rResult);

/**
 * SupNormAlt: Alternative implementation of SupNorm().
 * using SupPrefixClosed() instead of concatenating Sigma*.
 * SupNormAlt calculates the supremal sublanguage
 * of the closed language K (language generated by rK) that
 * is normal w.r.t. the closed language L (language generated 
 * by rL) and the set of observable events:
 * SupNormClosedAlt(K)wrt(L)=SupClosed(K-Pinv[P(L-K)])
 * 
 * SupNormAlt returns false on empty result.
 * todo: implement test routines, verify correctness
 * todo: check for efficient algorithm replacing above formula
 *
 * @param rL
 *   generates the closed language L=L(rL)
 * @param rOAlph
 *   observable alphabet
 * @param rK
 *   generates the closed language K=L(rK)
 * @param rResult
 *   generates the supremal normal and closed
 *   sublanguage, where Lm(rResult) is intersection of L(rResult) and Lm(rL)
 *   (marking consistent to rL)
 * 
 * @return
 *   true for nonempty result
 *
 * @exception Exception
 *   - Alphabets of generators don't match (id 500)
 *   - rOAlph not subset of rL.Alphabet() (id 506)
 *   - K is not subset of L. (id 0)
 * 
 *
 */
bool SupNormAlt(
  Generator& rL,
  const EventSet& rOAlph,
  const Generator& rK, 
  Generator& rResult);

/** rti wrapper (caution: args swap) */
void SupNorm(
  const vGenerator& rPlantGen, 
  const EventSet&  rOAlph,
  const vGenerator& rSpecGen, 
  vGenerator& rResGen);

/** rti wrapper (caution: args swap) */
void SupNorm(
  const cGenerator& rPlantGen, 
  const vGenerator& rSpecGen, 
  vGenerator& rResGen);

/** rti wrapper (caution: args swap) */
void SupNormClosed(
  const vGenerator& rPlantGen, 
  const EventSet&  rOAlph,
  const vGenerator& rSpecGen, 
  vGenerator& rResGen);

/** rti wrapper (caution: args swap) */
void SupNormClosed(
  const cGenerator& rPlantGen, 
  const vGenerator& rSpecGen, 
  vGenerator& rResGen);


} // end namespace
#endif
