/** @file ios_attributes.cpp I/O-system attributes */

/* 
   Standart IO Systems Plug-In for FAU Discrete Event Systems 
   Library (libfaudes)

   Copyright (C) 2010, Thomas Wittmann, Thomas Moor

*/


#include "ios_attributes.h"
#include <cfl_types.h>

namespace faudes {


/*
*********************************************************************************
*********************************************************************************
*********************************************************************************

Implementation: event attributes 

*********************************************************************************
*********************************************************************************
*********************************************************************************
*/


// faudes type std
FAUDES_TYPE_IMPLEMENTATION(Void,AttributeIosEvent,AttributeFlags)

  
// Assign my members
void AttributeIosEvent::DoAssign(const AttributeIosEvent& rSrcAttr) { 
  // call base (incl virtual clear)
  AttributeFlags::DoAssign(rSrcAttr);
}


//DoWrite(rTw,rLabel,pContext);
void AttributeIosEvent::DoWrite(TokenWriter& rTw, 
				const std::string& rLabel, 
				const Type* pContext) const 
{  
  (void) rLabel; 
  (void) pContext;
  // dont write default values
  if(IsDefault()) return;
  // debug output
  FD_DC("AttributeIosEvent(" << this << ")::DoWrite(tr)"); 
  Token token;	
  // check for invalid flags 
  if((mFlags & ~mAllAttributeIosEvent) == 0) {
    std::string option;
    // check flags
    switch (mFlags){
    case 0x10:
      option = "I";  
      break;
    case 0x20: 
      option = "O";
      break; 
    default:  
      option = "IO";
      break;
    }
    if(option!="") rTw.WriteOption(option);
  }
  // if other flags are set write hex-value of whole flag configuration
  else { 
    token.SetInteger16(mFlags);
    rTw << token;
  }
}



//DoRead(rTw,rLabel,pContext);
void AttributeIosEvent::DoRead(TokenReader& rTr, 
			       const std::string& rLabel, 
			       const Type* pContext) 
{    
  (void) rLabel;			
  (void) pContext;			
  // initialize with default value
  mFlags = mDefAttributeIosEvent;	 
  Token token;			
  rTr.Peek(token);			
  // if token indicates a non-valid flag
  // set mFlags to 0x12ff
  if(token.Type()==Token::Integer16) { 
      rTr.Get(token);
      mFlags=token.IntegerValue();
      return;
  }
  // if token indicates a valid option
  // determine its type and set flags
  if(token.IsOption()) { 
    rTr.Get(token);
    std::string option=token.OptionValue();
    if(option.find( 'I',0 ) != std::string::npos) 
      SetInput();
    if(option.find( 'O',0 ) != std::string::npos) 
      SetOutput();
    return;
  }
}


/*
*********************************************************************************
*********************************************************************************
*********************************************************************************

Implementation: state attributes 

*********************************************************************************
*********************************************************************************
*********************************************************************************
*/



// faudes type std
FAUDES_TYPE_IMPLEMENTATION(Void,AttributeIosState,AttributeFlags)

//DoAssign( rSrcAttr )
void AttributeIosState::DoAssign(const AttributeIosState& rSrcAttr) { 
  // call base (incl virtual clear)
  AttributeFlags::DoAssign(rSrcAttr);
}


//ToString()
std::string AttributeIosState::ToString(void) const {
  return ToStringInteger16(mFlags);
}


//DoWrite(rTw,rLabel,pContext);
void AttributeIosState::DoWrite(TokenWriter& rTw, 
				const std::string& rLabel, 
				const Type* pContext) const 
{
  (void) rLabel;	 
  (void) pContext;	
  // dont wrtite default value
  if(IsDefault()) return;
  FD_DC("AttributeIosState(" << this << ")::DoWrite(tr)");
  Token token;
  // check if only my flags are set
  if( (mFlags & ~mAllIosFlags) == 0 ) {
    std::string option="";      
    if(Input()) option = "I";
    if(Output()) option = "O";
    if(Error()) option = "E";
    token.SetOption(option);
  }
  // if other flags are used, write hex
  else 
    token.SetInteger16(mFlags);
  // do write
  rTw << token;
}


//DoRead(rTw,rLabel,pContext);
void AttributeIosState::DoRead(TokenReader& rTr, 
			       const std::string& rLabel, 
			       const Type* pContext) 
{
  (void) rLabel; 
  (void) pContext;
  // initialize with default value  
  mFlags=mDefIosFlags;
  // inspect token  
  Token token;
  rTr.Peek(token);
  // case a: its an hex number  
  if(token.Type()==Token::Integer16) { 
    rTr.Get(token);
    mFlags=token.IntegerValue();
    return;
  }
  // case b: its an option string
  if(token.IsOption()) {
    rTr.Get(token);
    std::string option=token.OptionValue();
    if(option.find( "I", 0) != std::string::npos) SetInput();
    if(option.find( "O", 0) != std::string::npos) SetOutput();
    if(option.find( "E", 0) != std::string::npos) SetError();
    return;
  }
}

}//end namespace faudes
