/** @file cfl_tokenreader.cpp @brief Class TokenReader */

/* FAU Discrete Event Systems Library (libfaudes)

Copyright (C) 2006  Bernd Opitz
Copyright (C) 2006  Thomas Moor
Exclusive copyright is granted to Klaus Schmidt

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */




#include "cfl_tokenreader.h"

namespace faudes {

// TokenReader(mode,instring)
TokenReader::TokenReader(Mode mode, const std::string& rInString)
  : mMode(mode), mpStream(NULL), mFileName("") 
{
  switch(mode) {
  case String: 
    // use mSStream
    FD_DV("TokenReader::Tokenreader(String, ...): " << rInString);
    mpSStream= new std::istringstream(rInString, std::istringstream::in | std::istringstream::binary);
    mpStream= mpSStream;
    Rewind();
    break;
  case File: 
    // set up mFStream
    FD_DV("TokenReader::Tokenreader(File, \"" << rInString <<"\")");
    mFStream.exceptions(std::ios::badbit|std::ios::failbit);
    try{
      mFStream.open(rInString.c_str(), std::ios::in | std::ios::binary); 
    } 
    catch (std::ios::failure&) {
      std::stringstream errstr;
      errstr << "Exception opening/reading file \""<< rInString << "\"";
      throw Exception("TokenReader::TokenReader", errstr.str(), 1);
    }
    mFileName=rInString;
    mpStream=&mFStream;
    Rewind();
    break;
  default:
    std::stringstream errstr;
    errstr << "Invalid Mode / Not implemented";
    throw Exception("TokenReader::TokenReader(mode,instring)", errstr.str(), 1);
  }
}


// TokenReader(rFilename)
TokenReader::TokenReader(const std::string& rFilename) 
  : mMode(File), mpStream(NULL), mFileName(rFilename) 
{
  // set up mFStream
  FD_DV("TokenReader::Tokenreader(File, \"" << rFilename <<"\")");
  mFStream.exceptions(std::ios::badbit|std::ios::failbit);
  try{
    mFStream.open(rFilename.c_str(), std::ios::in | std::ios::binary); 
  } 
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Exception opening/reading file \""<< rFilename << "\"";
    throw Exception("TokenReader::TokenReader", errstr.str(), 1);
  }
  mFileName=rFilename;
  mpStream=&mFStream;
  Rewind();
}


//  destruct
TokenReader::~TokenReader(void) {
  if(mMode==String) delete mpSStream;
}

// Rewind()
void TokenReader::Rewind(void) {
  FD_DV("TokenReader::Rewind: \"" << mFileName <<"\"");
  try {
    mpStream->clear();
    mpStream->seekg(0);
    mHasPeekToken=false;
    mPeekToken.SetNone();
    mLevel=0;
    mLevelPos.clear();
    mLevelPos.push_back(0);
    mLevelLine.clear();
    mLevelLine.push_back(1);
    mSeekLevel.clear();
    mLineCount=1;
    mFilePos=0;
  }
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Exception opening/reading file in "<< FileLine();
    throw Exception("TokenReader::Rewind", errstr.str(), 1);
  }
}


// FileName()
std::string TokenReader::FileName(void) const {
  return mFileName;
}

// Peek(token)
bool TokenReader::Peek(Token& token) {
  // fake end token for empty section
  if(!mHasPeekToken && mPeekToken.IsBegin() && mPeekToken.IsEnd()) {
    FD_DV("TokenReader: fake from : " << mPeekToken.Str());
    mPeekToken.SetEnd(std::string(mPeekToken.StringValue()));
    mHasPeekToken=true;
  }
  // get from peek buffer
  if(mHasPeekToken) { 
    token=mPeekToken;
    FD_DV("TokenReader: Peek: " << token.Str());
    return true;
  }
  // get new token
  try{
    mLineCount += token.Read(mpStream);
  } 
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Exception opening/reading file in "<< FileLine();
    throw Exception("TokenReader::Peek", errstr.str(), 1);
  }
  mPeekToken=token;
  FD_DV("TokenReader: Peek: " << token.Str());
  if(mPeekToken.IsNone()) {
    return false;
  } 
  mHasPeekToken=true;
  return true;
}

// Get(token)
bool TokenReader::Get(Token& token) {
  bool res;
  // get token from peek buffer and invalidate
  res=Peek(token);
  mHasPeekToken=false;
  // track state (level of nested sections, filepos etc)
  if(res) {
    mFilePos=mpStream->tellg();
    if(token.IsBegin()) {
      mLevel++;
      mLevelPos.push_back(mFilePos);
      mLevelLine.push_back(mLineCount);
    }
    if(token.IsEnd() && !token.IsBegin()) {
      mLevel--;
      if(mLevelPos.size()<=0 || mLevelLine.size()<=0) {
#ifdef FAUDES_CHECKED
        FD_WARN("TokenReader::Get(): Unbalanced end of section at " << FileLine());
#endif
	token.SetNone();
	return false;
      }
      mLevelPos.pop_back();
      mLevelLine.pop_back();
    }
  }
  FD_DV("TokenReader:Get(): " << token.Str());
  return res;
}

// SeekBegin(label)
void TokenReader::SeekBegin(const std::string& rLabel) {
  Token token;
  SeekBegin(rLabel,token);
}

// SeekBegin(label)
void TokenReader::SeekBegin(const std::string& rLabel,  Token& rToken) {
  // search for begin at any descending level, no rewind
  FD_DV("TokenReader::SeekBegin: " << rLabel << " at " << FileLine() << " level " << mLevel);
  int level=mLevel;
  long int startpos=mFilePos;
  int startline=mLineCount;
  for (;;) {
    // exception: did not get a token at all (incl. eof)
    if(!Peek(rToken)) {
      Rewind();
      std::stringstream errstr;
      errstr << "Section \"" << rLabel << "\" expected at " << FileLine() << " no more tokens";
      throw Exception("TokenReader::SeekBegin", errstr.str(), 51);
    }
    // exception: current section ends
    if((rToken.Type() == Token::End) && (mLevel == level)) {
      mpStream->seekg(startpos);
      mLineCount=startline;
      mHasPeekToken=false;
      mPeekToken.SetNone();
      std::stringstream errstr;
      errstr << "Section \"" << rLabel << "\" expected at " << FileLine() 
	     << "current section ended unexpected. Found: " << rToken.StringValue() << " Type " << rToken.Type();
      throw Exception("TokenReader::SeekBegin", errstr.str(), 51);
    }
    // success: found begin section
    if ((rToken.IsBegin()) && (rToken.StringValue() == rLabel))
      break;
    // go on seeking
    Get(rToken);
  }
}
 
// ReadBegin(label)
void TokenReader::ReadBegin(const std::string& rLabel) {
  Token token;
  ReadBegin(rLabel,token);
}

// ReadBegin(label,token)
void TokenReader::ReadBegin(const std::string& rLabel, Token& rToken) {
  FD_DV("Looking for Section \"" << rLabel << "\"");
  try {
    int level=mLevel;
    bool firstgo=true;
    long int startpos=mFilePos;
    FD_DV("section level " << level << " current pos " << startpos << " begin of section " << mLevelPos[level]);
    // search for begin at current level
    for (;;) {
      // exception: did not get a token at all (incl eof)
      if(!Peek(rToken)) {
	std::stringstream errstr;
	errstr << "Section \"" << rLabel << "\" expected at " << FileLine() << ", no token at all";
	throw Exception("TokenReader::ReadBegin Peek", errstr.str(), 51);
      }
      // success: found begin section
      if((rToken.IsBegin()) && (rToken.StringValue() == rLabel) && (mLevel==level)) {
	Get(rToken);
	break;
      }
      // rewind once when current section ends
      if(rToken.IsEnd() && !rToken.IsBegin() && (mLevel == level) && firstgo) {
	mpStream->seekg(mLevelPos[level]);
        mFilePos=mLevelPos[level];
	mLineCount=mLevelLine[level];                       
	firstgo=false;
	mHasPeekToken=false;
        mPeekToken.SetNone();
	continue;
      }
      // exception: did not find begin label
      if((mFilePos>=startpos) && (!firstgo)) {
	std::stringstream errstr;
	errstr << "Section \"" << rLabel << "\" expected at " << FileLine() << ", did not find begin label";
	throw Exception("TokenReader::ReadBegin Missed", errstr.str(), 51);
      }
      // skip this token
      Get(rToken);
    }
  }
  // catch my seek/tell errors
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Section \"" << rLabel << "\" expected at " << FileLine();
    throw Exception("TokenReader::ReadBegin Rewind", errstr.str(), 1);
  }
}
 

// ExistsBegin(label)
bool TokenReader::ExistsBegin(const std::string& rLabel) {
  FD_DV("TokenReader::ExistsBegin(): looking for Section \"" << rLabel << "\"");
  try {
    int level=mLevel;
    bool firstgo=true;
    long int startpos=mFilePos;
    FD_DV("section level " << level << " current pos " << startpos << " begin of section " << mLevelPos[level]);
    Token token;
    // search for begin at current level
    for(;;) {
      // fail: did not get a token at all (incl eof)
      if(!Peek(token)) {
        return false;
      }
      // success: found begin section
      if((token.IsBegin()) && (token.StringValue() == rLabel) && (mLevel==level)) {
	return true;
      }
      // rewind once when current section ends
      if((token.IsEnd() && !token.IsBegin()) && (mLevel == level) && firstgo) {
	mpStream->seekg(mLevelPos[level]);
        mFilePos=mLevelPos[level];
	mLineCount=mLevelLine[level];                       
	firstgo=false;
	mHasPeekToken=false;
        mPeekToken.SetNone();
	continue;
      }
      // fail: did not find begin label
      if((mFilePos>=startpos) && (!firstgo)) {
        return false;
      }
      // skip this token
      Get(token);
    }
  }
  // catch my seek/tell errors
  catch (std::ios::failure&) {
    std::stringstream errstr;
    errstr << "Section \"" << rLabel << "\" expected at " << FileLine();
    throw Exception("TokenReader::ReadBegin Rewind", errstr.str(), 1);
  }
  return false;
}
 
// ReadEnd(label)
void TokenReader::ReadEnd(const std::string& rLabel) {
  FD_DV("TokenReader::ReadEnd: " << rLabel << " at " << FileLine() );
  // search for end at current level
  int level=mLevel;
  Token token;
  for (;;) {
    // exception: did not get a token at all
    if(!Get(token)) {
      std::stringstream errstr;
      errstr << "end of Section \"" << rLabel << "\" expected at " << FileLine();
      throw Exception("TokenReader::ReadEnd", errstr.str(), 51);
    }
    // success: found end of current section
    if(token.IsEnd() && !token.IsBegin() && (token.StringValue() == rLabel) && (mLevel==level-1)) {
      break;
    }
    // exception: current section ends with unexpected label
    if(mLevel<level) {
      std::stringstream errstr;
      errstr << "end of Section \"" << rLabel << "\" expected at " << FileLine();
      throw Exception("TokenReader::ReadEnd", errstr.str(), 51);
    }
  }
}

// Recover()
bool TokenReader::Recover(int level) {
  // trivial cases
  if(level>mLevel) return false;
  if(level==mLevel) return true;
  // loop until match
  Token token;
  while(Get(token)) 
    if(mLevel<=level) break;
  // done
  return level==mLevel;
}

// Eos(label)
bool TokenReader::Eos(const std::string& rLabel) {
  // peek token and check for end of section
  Token token;
  Peek(token);
  if(! (token.IsEnd() && !token.IsBegin()))
    return false;
  if((token.IsEnd() && !token.IsBegin())  && (token.StringValue() == rLabel))
    return true; 
  std::stringstream errstr;
  errstr << "Section End\"" << rLabel << "\" expected at " << FileLine();
  throw Exception("TokenReader::Eos", errstr.str(), 51);
  return false;
}
  

// ReadInteger()
long int TokenReader::ReadInteger(void) {
  Token token;			
  Get(token);
  if(!token.IsInteger()) {
    std::stringstream errstr;
    errstr << "Integer expected at " << FileLine();
    throw Exception("TokenReader::TokenReader", errstr.str(), 50);
  }
  return token.IntegerValue();
}

// ReadFloat()
double TokenReader::ReadFloat(void) {
  Token token;			
  Get(token);
  if((!token.IsFloat()) && (!token.IsInteger())) {
    std::stringstream errstr;
    errstr << "Float expected at " << FileLine();
    throw Exception("TokenReader::TokenReader", errstr.str(), 50);
  }
  return token.FloatValue();
}

// ReadString()
const std::string& TokenReader::ReadString(void) {
  Token token;			
  Get(token);
  if(!token.IsString()) {
    std::stringstream errstr;
    errstr << "Name expected at " << FileLine();
    throw Exception("TokenReader::TokenReader", errstr.str(), 50);
  }
  mLastString=token.StringValue();
  return(mLastString);
}

 
// ReadOption()
const std::string& TokenReader::ReadOption(void) {
  Token token;			
  Get(token);
  if(!token.IsOption()) {
    std::stringstream errstr;
    errstr << "Option expected at " << FileLine();
    throw Exception("TokenReader::TokenReader", errstr.str(), 50);
  }
  mLastString=token.OptionValue();
  return(mLastString);
}

// ReadBinary()
const std::string& TokenReader::ReadBinary(void) {
  Token token;			
  Get(token);
  if(!token.IsBinary()) {
    std::stringstream errstr;
    errstr << "Binary string expected at " << FileLine();
    throw Exception("TokenReader::TokenReader", errstr.str(), 50);
  }
  mLastString=token.StringValue();
  return(mLastString);
}


// ReadText()
const std::string& TokenReader::ReadText(void) {
  // do my own reading
  int ll=Token::ReadEscapedString(mpStream,'<',mLastString);
  if(ll<0) {
    std::stringstream errstr;
    errstr << "Text expected at " << FileLine();
    throw Exception("TokenReader::TokenReader", errstr.str(), 50);
  }
  // strip leading/tailing space
  static const std::string white=" \n\t\r\v\f";
  unsigned int pos1=mLastString.find_first_not_of(white);
  if(pos1!=std::string::npos)  
    mLastString=mLastString.substr(pos1);
  else
    mLastString.clear();
  unsigned int pos2=mLastString.find_last_not_of(white);
  if(pos2!=std::string::npos)  
    mLastString.erase(pos2+1);
  // clear peak buffer
  mHasPeekToken=false;
  mPeekToken.SetNone();
  // do linecount
  mLineCount+=ll;
  return mLastString;
}

 

// Line()
int TokenReader::Line(void) const {
  return mLineCount;
}

// FileLine()
std::string TokenReader::FileLine(void) const {
  if(mFileName!="")
    return "("+ mFileName + ":" + ToStringInteger(mLineCount) +")";
  else
    return "(#" + ToStringInteger(mLineCount) +")";
}

} // namespace faudes
