/************************************************
 ************************************************
 ************************************************/

/**
 
@mainpage libFAUDES C++ API


<p>
As a C++ library, the natural application interface of libFAUDES is given by C++ class and
function declarations. The C++ API documentation is generated directly from the C++ source code 
and supplemented by <a href="group__Tutorials.html">tutorials</a> that illustrate the intended
ussage by simple example programs. 
</p>
<p>
The C++ API is organized as follows:
<ul class="l2">
<li><p>
<a href="group__ContainerClasses.html">container classes</a> 
for sets of events, states and transitions; 
</p></li>
<li><p>
<a href="group__GeneratorClasses.html">generator classes</a> 
to model finite state machines; 
</p></li>
<li><p>
<a href="group__GeneratorFunctions.html">functions</a> 
that implement general purpose algorithms on generators; and 
</p></li>
<li><p>
<a href="group__AllPlugins.html">plug-ins</a> 
to provide specialised algorithms and derived generator classes.
</p></li>
<li><p>
<a href="group__RunTimeInterface.html">run-time interface (RTI)</a> 
for transparent extensions (advanced topic)
<li><p>
<a href="group__TokenIO.html">token-IO</a> 
to serialize faudes typed objects (advanced topic)
</p></li>
</ul>
</p>


<p>
In addition to the C++ API, libFAUDES provides a 
<a href="../registry/index.html">user-reference</a> to address the typical user 
of a lobFAUDES application without bothering with C++ implementation technicalities.
</p>


<br>

@section SecHomeCompile Compile


<p>
libFAUDES is coded in posix C++ and should compile within any recent C++ 
development environment. The supplied
<tt>Makefile</tt> is targeted for the GNU-Tool-Chain and provides the
following build targets:
</p>

<ul class="l2">
<li><p>
<tt>make default</tt> to build shared objects, static archives and utility executables;
</p></li>
<li><p>
<tt>make tutorial</tt> to compile examples from the tutorial section.
</p></li>
<li><p>
<tt>make clean</tt> to remove temporary object files
</p></li>
</ul>

<p>
For more information on how to compile libFAUDES,
see the 
<a href="../faudes_build.html">build-system documentation</a>.
</p>

<br>


@section SecCoding Coding Conventions


<p>
In order to ease communication, libFAUDES sources try 
to follow some notational conventions. 
They are extracted from 
<a href="http://www.possibility.com/Cpp/CppCodingStandard.html" target="_top">
www.possibility.com/Cpp/CppCodingStandard.html</a>.
</p>

<ul class="l2">
<li><p>
member variable names are upper case  with lower case prefix that
indicates ownership, i.e. 
<tt>m</tt> for ordinary members,
<tt>p</tt> for pointer to variables controlled somewhere else,
<tt>r</tt> for refernces to variables controlled somewhere else, and 
<tt>mp</tt> for pointers to variables controlled here;
</p></li>
<li><p>
function-local stack variables are lower case,  
</p></li>
<li><p>
class names are upper case, with prefix T for class templates.
</p></li>
</ul>

<p>
Moreover, we
</p>
<ul class="l2">
<li><p>
avoid the Set/Get prefixes of members functions and use overloading
instead;
</p></li>
<li><p>
throw exceptions rather than abort() or assert();
</p></li>
<li><p>
use FAUDES_filename_H macro in headers;
</p></li>
<li><p>
use the provided macros in cfl_definitions FD_WARN etc for any console output;
</p></li>
<li><p>
use 2-space indent to have shorter lines of code.
</p></li>
</ul>

<br>

@section SecHomeLicense License 


<p>
The core libFAUDES sources incl. the C++ API 
are provided for free under conditions of the GNU Lesser General Public License. 
Note that different and perhaps more restrictive license terms may apply to the plug-ins that reside in the plug-in directory.
</p>




<br>
<br>
<br>


<p>
Copyright (C) 2006 Bernd Opitz <br>
Copyright (C) 2008, 2010 Thomas Moor, Klaus Schmidt, Sebastian Perk <br>
</p>
*/

/************************************************
 ************************************************
 ************************************************/


/** @defgroup GeneratorClasses  Generator Classes

A generator is a tupel G = (Q, Sigma, Delta, Qo, Qm). Generators are used
to represent the closed and the marked language L(G) 
and L_m(G) respectively. The core members of FAUDS generator classes are
- mpStates, the stateset Q, type faudes::StateSet;
- mpAlphabet,  the Alphabet Sigma, type faudes::EventSet;
- mpTransRel, the transitionrelation Delta, type faudes::TransSet; 
- mInitStates, the set of initial states Qo, type faudes::StateSet;
- mMarkedStates, the set of marked states Qm, type faudes::StateSet.

The base class for all libFAUDES generator classes is the faudes::vGenerator. It provides
functions for read and write access to the core members and file IO. Derived generator classes
providing attibute attached to events, states and transitions in order to
extend basic generator semantics. Examples for attributes are controllability flags for
events, addressed by the cGenerator.

*/

/************************************************
 ************************************************
 ************************************************/

/** @defgroup ContainerClasses State, Event and Transition Set

This module collects all basic container classes aka state- event- 
and transition-sets.

*/

/************************************************
 ************************************************
 ************************************************/

/** @defgroup GeneratorFunctions  General Purpose Functions

This module collects general purpose functions on Generators. 
It includes functions related to regular expressions,
projection, parallel composition and basic controller synthesis.

*/


/************************************************
 ************************************************
 ************************************************/

/** @defgroup Tutorials  Tutorials

This module collects all tutorials. The tutorials are seen as a pragmatic
introduction to supplement the doxygen generated html documentaion of 
class and function definitions. Plug-Ins are meant to contribute additional tutorials 
to illustrate their intended usage.  

To compile the tutorials call make with target &quot;tutorial&quot; 
in the &quot;libfaudes&quot; directory of the source tree:
<pre>$ make tutorial
</pre>


*/

/************************************************
 ************************************************
 ************************************************/


/** @defgroup AllPlugins PlugIns

Find below a list of all plug-ins that are installed and enabled
in this copy of libFAUDES.

To install a plug-in, copy its sources into the plug directory
of the libfaudes source tree. To enable a plug-in, either edit the
main Makefile or add its name to the FAUDES_PLUGINS environment variable.
The <a href="../faudes_algorithm_example.html">example plug-in</a> 
further illustrates the plug-in infrastructure.

For instructions on how to compile libFAUDES, see the 
<a href="../faudes_build.html">build-system</a> documentation.

*/

/************************************************
 ************************************************
 ************************************************/

/** @defgroup TokenIO  Faudes object serialization.

Objects of a class derived from the univarsal base faudes::Type
provide a uniform interface for serialization. 
It includes reading and writing the objects
configuration data from/to a file or a std::string; see faudes::Type 
for the actual interface.

Serialization is implemented by three classes
- the faudes::TokenReader class to read tokenized character stream from file or string
- the faudes::TokenWriter class to write tokenized character stream to file or string
- the faudes::Token class to model atomic data units like integers, sections tags and strings.

A token is either of principle type string, number, or tag. 
Tags indicate the beginning or end of a section, must balance, and are XML comliant.
The character data inbeween tags is interpreted as a sequence of strings and numbers,
to e.g. conveniently represent a list of events, transition or states. While the Token class
implements reading and writing tokens from a C++ stream, the TokenReader and TokenWriter
classes keep track of sections. Thus, opening a file via a TokenReader allows to scan for
a particular tag or to loop over all tokens from a particular section.


<b>Technical Note: new XML format</b>

Up to libFAUDES 2.16b, the actual stream format was intended to be 
human editable, and this will continue to be the case for generators and alphabets. 
However, since the first design of libFAUDES, a number of other faudes types where inroduced, mainly to
configure the run-time behaviour of tools like the simulator or hardware access.
For the latter kind of objects, we feel that a more rigorous XML format is more apropriate, since
this enables the ussage of professional XML editors for configuration and input-data maintenance. 
Thus, in 2.16f we started to implement an alternative serialization format which will coexist 
with the old format for some time. During this transitional phase, libFAUDES will read
both formats and use the new format as a default for output (exept for generators and alphabets,
which will continue use the original human editable format as a default).
The transitional phase is expected be completed until libFAUDES 2.20, 
the utility <tt>libfaudes/bin/fts2ftx</tt> is supplied for file conversion. 



*/

/************************************************
 ************************************************
 ************************************************/

/** @namespace faudes

libFAUDES resides within the namespace faudes. Plug-Ins may use the same namespace.

*/
